@@ -31,6 +31,8 @@ use util::byte_utils;
 use util::logger::Logger;
 use util::ser::{Readable, Writer, Writeable};
 
+use io;
+use prelude::*;
 use core::cmp;
 use core::mem;
 use core::ops::Deref;
@@ -87,14 +89,14 @@ impl RevokedOutput {
 }
 
 impl_writeable_tlv_based!(RevokedOutput, {
-	(0, per_commitment_point),
-	(2, counterparty_delayed_payment_base_key),
-	(4, counterparty_htlc_base_key),
-	(6, per_commitment_key),
-	(8, weight),
-	(10, amount),
-	(12, on_counterparty_tx_csv),
-}, {}, {});
+	(0, per_commitment_point, required),
+	(2, counterparty_delayed_payment_base_key, required),
+	(4, counterparty_htlc_base_key, required),
+	(6, per_commitment_key, required),
+	(8, weight, required),
+	(10, amount, required),
+	(12, on_counterparty_tx_csv, required),
+});
 
 /// A struct to describe a revoked offered output and corresponding information to generate a
 /// solving witness.
@@ -131,14 +133,14 @@ impl RevokedHTLCOutput {
 }
 
 impl_writeable_tlv_based!(RevokedHTLCOutput, {
-	(0, per_commitment_point),
-	(2, counterparty_delayed_payment_base_key),
-	(4, counterparty_htlc_base_key),
-	(6, per_commitment_key),
-	(8, weight),
-	(10, amount),
-	(12, htlc),
-}, {}, {});
+	(0, per_commitment_point, required),
+	(2, counterparty_delayed_payment_base_key, required),
+	(4, counterparty_htlc_base_key, required),
+	(6, per_commitment_key, required),
+	(8, weight, required),
+	(10, amount, required),
+	(12, htlc, required),
+});
 
 /// A struct to describe a HTLC output on a counterparty commitment transaction.
 ///
@@ -168,12 +170,12 @@ impl CounterpartyOfferedHTLCOutput {
 }
 
 impl_writeable_tlv_based!(CounterpartyOfferedHTLCOutput, {
-	(0, per_commitment_point),
-	(2, counterparty_delayed_payment_base_key),
-	(4, counterparty_htlc_base_key),
-	(6, preimage),
-	(8, htlc),
-}, {}, {});
+	(0, per_commitment_point, required),
+	(2, counterparty_delayed_payment_base_key, required),
+	(4, counterparty_htlc_base_key, required),
+	(6, preimage, required),
+	(8, htlc, required),
+});
 
 /// A struct to describe a HTLC output on a counterparty commitment transaction.
 ///
@@ -199,11 +201,11 @@ impl CounterpartyReceivedHTLCOutput {
 }
 
 impl_writeable_tlv_based!(CounterpartyReceivedHTLCOutput, {
-	(0, per_commitment_point),
-	(2, counterparty_delayed_payment_base_key),
-	(4, counterparty_htlc_base_key),
-	(6, htlc),
-}, {}, {});
+	(0, per_commitment_point, required),
+	(2, counterparty_delayed_payment_base_key, required),
+	(4, counterparty_htlc_base_key, required),
+	(6, htlc, required),
+});
 
 /// A struct to describe a HTLC output on holder commitment transaction.
 ///
@@ -236,11 +238,10 @@ impl HolderHTLCOutput {
 }
 
 impl_writeable_tlv_based!(HolderHTLCOutput, {
-	(0, amount),
-	(2, cltv_expiry),
-}, {
-	(4, preimage),
-}, {});
+	(0, amount, required),
+	(2, cltv_expiry, required),
+	(4, preimage, option)
+});
 
 /// A struct to describe the channel output on the funding transaction.
 ///
@@ -259,8 +260,8 @@ impl HolderFundingOutput {
 }
 
 impl_writeable_tlv_based!(HolderFundingOutput, {
-	(0, funding_redeemscript),
-}, {}, {});
+	(0, funding_redeemscript, required),
+});
 
 /// A wrapper encapsulating all in-protocol differing outputs types.
 ///
@@ -396,8 +397,8 @@ impl PackageSolvingData {
 			PackageSolvingData::RevokedOutput(_) => output_conf_height + 1,
 			PackageSolvingData::RevokedHTLCOutput(_) => output_conf_height + 1,
 			PackageSolvingData::CounterpartyOfferedHTLCOutput(_) => output_conf_height + 1,
-			PackageSolvingData::CounterpartyReceivedHTLCOutput(ref outp) => std::cmp::max(outp.htlc.cltv_expiry, output_conf_height + 1),
-			PackageSolvingData::HolderHTLCOutput(ref outp) => std::cmp::max(outp.cltv_expiry, output_conf_height + 1),
+			PackageSolvingData::CounterpartyReceivedHTLCOutput(ref outp) => cmp::max(outp.htlc.cltv_expiry, output_conf_height + 1),
+			PackageSolvingData::HolderHTLCOutput(ref outp) => cmp::max(outp.cltv_expiry, output_conf_height + 1),
 			PackageSolvingData::HolderFundingOutput(_) => output_conf_height + 1,
 		};
 		absolute_timelock
@@ -603,18 +604,18 @@ impl PackageTemplate {
 					});
 				}
 				for (i, (outpoint, out)) in self.inputs.iter().enumerate() {
-					log_trace!(logger, "Adding claiming input for outpoint {}:{}", outpoint.txid, outpoint.vout);
+					log_debug!(logger, "Adding claiming input for outpoint {}:{}", outpoint.txid, outpoint.vout);
 					if !out.finalize_input(&mut bumped_tx, i, onchain_handler) { return None; }
 				}
-				log_trace!(logger, "Finalized transaction {} ready to broadcast", bumped_tx.txid());
+				log_debug!(logger, "Finalized transaction {} ready to broadcast", bumped_tx.txid());
 				return Some(bumped_tx);
 			},
 			PackageMalleability::Untractable => {
 				debug_assert_eq!(value, 0, "value is ignored for non-malleable packages, should be zero to ensure callsites are correct");
 				if let Some((outpoint, outp)) = self.inputs.first() {
 					if let Some(final_tx) = outp.get_finalized_tx(outpoint, onchain_handler) {
-						log_trace!(logger, "Adding claiming input for outpoint {}:{}", outpoint.txid, outpoint.vout);
-						log_trace!(logger, "Finalized transaction {} ready to broadcast", final_tx.txid());
+						log_debug!(logger, "Adding claiming input for outpoint {}:{}", outpoint.txid, outpoint.vout);
+						log_debug!(logger, "Finalized transaction {} ready to broadcast", final_tx.txid());
 						return Some(final_tx);
 					}
 					return None;
@@ -683,23 +684,24 @@ impl PackageTemplate {
 }
 
 impl Writeable for PackageTemplate {
-	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), ::std::io::Error> {
+	fn write<W: Writer>(&self, writer: &mut W) -> Result<(), io::Error> {
 		writer.write_all(&byte_utils::be64_to_array(self.inputs.len() as u64))?;
 		for (ref outpoint, ref rev_outp) in self.inputs.iter() {
 			outpoint.write(writer)?;
 			rev_outp.write(writer)?;
 		}
 		write_tlv_fields!(writer, {
-			(0, self.soonest_conf_deadline),
-			(2, self.feerate_previous),
-			(4, self.height_original),
-		}, { (6, self.height_timer) });
+			(0, self.soonest_conf_deadline, required),
+			(2, self.feerate_previous, required),
+			(4, self.height_original, required),
+			(6, self.height_timer, option)
+		});
 		Ok(())
 	}
 }
 
 impl Readable for PackageTemplate {
-	fn read<R: ::std::io::Read>(reader: &mut R) -> Result<Self, DecodeError> {
+	fn read<R: io::Read>(reader: &mut R) -> Result<Self, DecodeError> {
 		let inputs_count = <u64 as Readable>::read(reader)?;
 		let mut inputs: Vec<(BitcoinOutPoint, PackageSolvingData)> = Vec::with_capacity(cmp::min(inputs_count as usize, MAX_ALLOC_SIZE / 128));
 		for _ in 0..inputs_count {
@@ -722,10 +724,11 @@ impl Readable for PackageTemplate {
 		let mut height_timer = None;
 		let mut height_original = 0;
 		read_tlv_fields!(reader, {
-			(0, soonest_conf_deadline),
-			(2, feerate_previous),
-			(4, height_original)
-		}, { (6, height_timer) });
+			(0, soonest_conf_deadline, required),
+			(2, feerate_previous, required),
+			(4, height_original, required),
+			(6, height_timer, option),
+		});
 		Ok(PackageTemplate {
 			inputs,
 			malleability,
@@ -794,13 +797,13 @@ fn feerate_bump<F: Deref, L: Deref>(predicted_weight: usize, input_amounts: u64,
 			// ...else just increase the previous feerate by 25% (because that's a nice number)
 			let new_fee = previous_feerate * (predicted_weight as u64) / 750;
 			if input_amounts <= new_fee {
-				log_trace!(logger, "Can't 25% bump new claiming tx, amount {} is too small", input_amounts);
+				log_warn!(logger, "Can't 25% bump new claiming tx, amount {} is too small", input_amounts);
 				return None;
 			}
 			new_fee
 		}
 	} else {
-		log_trace!(logger, "Can't new-estimation bump new claiming tx, amount {} is too small", input_amounts);
+		log_warn!(logger, "Can't new-estimation bump new claiming tx, amount {} is too small", input_amounts);
 		return None;
 	};
 
