@@ -15,8 +15,9 @@ use chain::channelmonitor::ChannelMonitor;
 use chain::transaction::OutPoint;
 use ln::{PaymentPreimage, PaymentHash, PaymentSecret};
 use ln::channelmanager::{ChainParameters, ChannelManager, ChannelManagerReadArgs, RAACommitmentOrder, PaymentSendFailure};
-use routing::router::{Route, get_route};
 use routing::network_graph::{NetGraphMsgHandler, NetworkGraph};
+use routing::router::{Route, get_route};
+use routing::scorer::Scorer;
 use ln::features::{InitFeatures, InvoiceFeatures};
 use ln::msgs;
 use ln::msgs::{ChannelMessageHandler,RoutingMessageHandler};
@@ -274,10 +275,9 @@ impl<'a, 'b, 'c> Drop for Node<'a, 'b, 'c> {
 			let feeest = test_utils::TestFeeEstimator { sat_per_kw: Mutex::new(253) };
 			let mut deserialized_monitors = Vec::new();
 			{
-				let old_monitors = self.chain_monitor.chain_monitor.monitors.read().unwrap();
-				for (_, old_monitor) in old_monitors.iter() {
+				for outpoint in self.chain_monitor.chain_monitor.list_monitors() {
 					let mut w = test_utils::TestVecWriter(Vec::new());
-					old_monitor.write(&mut w).unwrap();
+					self.chain_monitor.chain_monitor.get_monitor(outpoint).unwrap().write(&mut w).unwrap();
 					let (_, deserialized_monitor) = <(BlockHash, ChannelMonitor<EnforcingSigner>)>::read(
 						&mut io::Cursor::new(&w.0), self.keys_manager).unwrap();
 					deserialized_monitors.push(deserialized_monitor);
@@ -437,20 +437,35 @@ macro_rules! get_feerate {
 	}
 }
 
+/// Returns a channel monitor given a channel id, making some naive assumptions
+#[macro_export]
+macro_rules! get_monitor {
+	($node: expr, $channel_id: expr) => {
+		{
+			use bitcoin::hashes::Hash;
+			let mut monitor = None;
+			// Assume funding vout is either 0 or 1 blindly
+			for index in 0..2 {
+				if let Ok(mon) = $node.chain_monitor.chain_monitor.get_monitor(
+					$crate::chain::transaction::OutPoint {
+						txid: bitcoin::Txid::from_slice(&$channel_id[..]).unwrap(), index
+					})
+				{
+					monitor = Some(mon);
+					break;
+				}
+			}
+			monitor.unwrap()
+		}
+	}
+}
+
 /// Returns any local commitment transactions for the channel.
 #[macro_export]
 macro_rules! get_local_commitment_txn {
 	($node: expr, $channel_id: expr) => {
 		{
-			let monitors = $node.chain_monitor.chain_monitor.monitors.read().unwrap();
-			let mut commitment_txn = None;
-			for (funding_txo, monitor) in monitors.iter() {
-				if funding_txo.to_channel_id() == $channel_id {
-					commitment_txn = Some(monitor.unsafe_get_latest_holder_commitment_txn(&$node.logger));
-					break;
-				}
-			}
-			commitment_txn.unwrap()
+			$crate::get_monitor!($node, $channel_id).unsafe_get_latest_holder_commitment_txn(&$node.logger)
 		}
 	}
 }
@@ -512,11 +527,12 @@ pub fn create_funding_transaction<'a, 'b, 'c>(node: &Node<'a, 'b, 'c>, expected_
 }
 
 pub fn create_chan_between_nodes_with_value_init<'a, 'b, 'c>(node_a: &Node<'a, 'b, 'c>, node_b: &Node<'a, 'b, 'c>, channel_value: u64, push_msat: u64, a_flags: InitFeatures, b_flags: InitFeatures) -> Transaction {
-	node_a.node.create_channel(node_b.node.get_our_node_id(), channel_value, push_msat, 42, None).unwrap();
+	let create_chan_id = node_a.node.create_channel(node_b.node.get_our_node_id(), channel_value, push_msat, 42, None).unwrap();
 	node_b.node.handle_open_channel(&node_a.node.get_our_node_id(), a_flags, &get_event_msg!(node_a, MessageSendEvent::SendOpenChannel, node_b.node.get_our_node_id()));
 	node_a.node.handle_accept_channel(&node_b.node.get_our_node_id(), b_flags, &get_event_msg!(node_b, MessageSendEvent::SendAcceptChannel, node_a.node.get_our_node_id()));
 
 	let (temporary_channel_id, tx, funding_output) = create_funding_transaction(node_a, channel_value, 42);
+	assert_eq!(temporary_channel_id, create_chan_id);
 
 	node_a.node.funding_transaction_generated(&temporary_channel_id, tx.clone()).unwrap();
 	check_added_monitors!(node_a, 0);
@@ -763,21 +779,29 @@ macro_rules! check_closed_broadcast {
 	}}
 }
 
-/// Check that a channel's closing channel event has been issued
+/// Check that a channel's closing channel events has been issued
 #[macro_export]
 macro_rules! check_closed_event {
-	($node: expr, $events: expr, $reason: expr) => {{
+	($node: expr, $events: expr, $reason: expr) => {
+		check_closed_event!($node, $events, $reason, false);
+	};
+	($node: expr, $events: expr, $reason: expr, $is_check_discard_funding: expr) => {{
 		let events = $node.node.get_and_clear_pending_events();
 		assert_eq!(events.len(), $events);
 		let expected_reason = $reason;
+		let mut issues_discard_funding = false;
 		for event in events {
 			match event {
 				Event::ChannelClosed { ref reason, .. } => {
 					assert_eq!(*reason, expected_reason);
 				},
+				Event::DiscardFunding { .. } => {
+					issues_discard_funding = true;
+				}
 				_ => panic!("Unexpected event"),
 			}
 		}
+		assert_eq!($is_check_discard_funding, issues_discard_funding);
 	}}
 }
 
@@ -965,10 +989,16 @@ macro_rules! commitment_signed_dance {
 macro_rules! get_payment_preimage_hash {
 	($dest_node: expr) => {
 		{
-			let payment_preimage = PaymentPreimage([*$dest_node.network_payment_count.borrow(); 32]);
-			*$dest_node.network_payment_count.borrow_mut() += 1;
+			get_payment_preimage_hash!($dest_node, None)
+		}
+	};
+	($dest_node: expr, $min_value_msat: expr) => {
+		{
+			let mut payment_count = $dest_node.network_payment_count.borrow_mut();
+			let payment_preimage = PaymentPreimage([*payment_count; 32]);
+			*payment_count += 1;
 			let payment_hash = PaymentHash(Sha256::hash(&payment_preimage.0[..]).into_inner());
-			let payment_secret = $dest_node.node.create_inbound_payment_for_hash(payment_hash, None, 7200, 0).unwrap();
+			let payment_secret = $dest_node.node.create_inbound_payment_for_hash(payment_hash, $min_value_msat, 7200, 0).unwrap();
 			(payment_preimage, payment_hash, payment_secret)
 		}
 	}
@@ -977,11 +1007,18 @@ macro_rules! get_payment_preimage_hash {
 #[cfg(test)]
 macro_rules! get_route_and_payment_hash {
 	($send_node: expr, $recv_node: expr, $recv_value: expr) => {{
-		let (payment_preimage, payment_hash, payment_secret) = get_payment_preimage_hash!($recv_node);
+		get_route_and_payment_hash!($send_node, $recv_node, vec![], $recv_value, TEST_FINAL_CLTV)
+	}};
+	($send_node: expr, $recv_node: expr, $last_hops: expr, $recv_value: expr, $cltv: expr) => {{
+		let (payment_preimage, payment_hash, payment_secret) = get_payment_preimage_hash!($recv_node, Some($recv_value));
 		let net_graph_msg_handler = &$send_node.net_graph_msg_handler;
-		let route = get_route(&$send_node.node.get_our_node_id(),
-			&net_graph_msg_handler.network_graph,
-			&$recv_node.node.get_our_node_id(), None, None, &Vec::new(), $recv_value, TEST_FINAL_CLTV, $send_node.logger).unwrap();
+		let scorer = ::routing::scorer::Scorer::new(0);
+		let route = ::routing::router::get_route(
+			&$send_node.node.get_our_node_id(), &net_graph_msg_handler.network_graph,
+			&$recv_node.node.get_our_node_id(), Some(::ln::features::InvoiceFeatures::known()),
+			Some(&$send_node.node.list_usable_channels().iter().collect::<Vec<_>>()),
+			&$last_hops, $recv_value, $cltv, $send_node.logger, &scorer
+		).unwrap();
 		(route, payment_hash, payment_preimage, payment_secret)
 	}}
 }
@@ -1043,10 +1080,12 @@ macro_rules! expect_payment_received {
 macro_rules! expect_payment_sent {
 	($node: expr, $expected_payment_preimage: expr) => {
 		let events = $node.node.get_and_clear_pending_events();
+		let expected_payment_hash = PaymentHash(Sha256::hash(&$expected_payment_preimage.0).into_inner());
 		assert_eq!(events.len(), 1);
 		match events[0] {
-			Event::PaymentSent { ref payment_preimage } => {
+			Event::PaymentSent { ref payment_preimage, ref payment_hash } => {
 				assert_eq!($expected_payment_preimage, *payment_preimage);
+				assert_eq!(expected_payment_hash, *payment_hash);
 			},
 			_ => panic!("Unexpected event"),
 		}
@@ -1289,10 +1328,11 @@ pub const TEST_FINAL_CLTV: u32 = 70;
 
 pub fn route_payment<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>, expected_route: &[&Node<'a, 'b, 'c>], recv_value: u64) -> (PaymentPreimage, PaymentHash, PaymentSecret) {
 	let net_graph_msg_handler = &origin_node.net_graph_msg_handler;
+	let scorer = Scorer::new(0);
 	let route = get_route(&origin_node.node.get_our_node_id(), &net_graph_msg_handler.network_graph,
 		&expected_route.last().unwrap().node.get_our_node_id(), Some(InvoiceFeatures::known()),
 		Some(&origin_node.node.list_usable_channels().iter().collect::<Vec<_>>()), &[],
-		recv_value, TEST_FINAL_CLTV, origin_node.logger).unwrap();
+		recv_value, TEST_FINAL_CLTV, origin_node.logger, &scorer).unwrap();
 	assert_eq!(route.paths.len(), 1);
 	assert_eq!(route.paths[0].len(), expected_route.len());
 	for (node, hop) in expected_route.iter().zip(route.paths[0].iter()) {
@@ -1304,7 +1344,8 @@ pub fn route_payment<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>, expected_route:
 
 pub fn route_over_limit<'a, 'b, 'c>(origin_node: &Node<'a, 'b, 'c>, expected_route: &[&Node<'a, 'b, 'c>], recv_value: u64)  {
 	let net_graph_msg_handler = &origin_node.net_graph_msg_handler;
-	let route = get_route(&origin_node.node.get_our_node_id(), &net_graph_msg_handler.network_graph, &expected_route.last().unwrap().node.get_our_node_id(), Some(InvoiceFeatures::known()), None, &Vec::new(), recv_value, TEST_FINAL_CLTV, origin_node.logger).unwrap();
+	let scorer = Scorer::new(0);
+	let route = get_route(&origin_node.node.get_our_node_id(), &net_graph_msg_handler.network_graph, &expected_route.last().unwrap().node.get_our_node_id(), Some(InvoiceFeatures::known()), None, &Vec::new(), recv_value, TEST_FINAL_CLTV, origin_node.logger, &scorer).unwrap();
 	assert_eq!(route.paths.len(), 1);
 	assert_eq!(route.paths[0].len(), expected_route.len());
 	for (node, hop) in expected_route.iter().zip(route.paths[0].iter()) {
